@@ -18,6 +18,9 @@ except:
 #import pandas as pd
 
 
+from scipy.stats.mstats import f_oneway as aov
+from scipy.stats import ttest_ind as ttest
+
 ##############################################################
 
 ##########################################################################
@@ -327,17 +330,17 @@ if not args.hdf5:
 
 ################################
 ## ray functions
-def get_indices(threads, num_genes):
-    indices_list = []
-    for t in range(threads):
-        indices_list.append([])
-    temp_idx = 0
-    while temp_idx < num_genes:
-        for t in range(threads):
-            if temp_idx < num_genes:
-                indices_list[t].append(temp_idx)
-                temp_idx += 1
-    return(indices_list)
+# def get_indices(threads, num_genes):
+#     indices_list = []
+#     for t in range(threads):
+#         indices_list.append([])
+#     temp_idx = 0
+#     while temp_idx < num_genes:
+#         for t in range(threads):
+#             if temp_idx < num_genes:
+#                 indices_list[t].append(temp_idx)
+#                 temp_idx += 1
+#     return(indices_list)
 
 
 def get_num_rows_from_dict_lists(dict_list):
@@ -375,6 +378,115 @@ def ray_dicts_to_array(dict_list):
     return(out_array)
 
 
+@ray.remote
+def ray_get_global_cell_type_aov_stats(indices, 
+                                       ray_full_expression,
+                                       ray_list_of_k_sample_indices,
+                                       hdf5_file = None):
+    max_index = max(indices)
+    min_index = min(indices)
+
+    ## make the ray_list_of_k_sample_indices arrays
+    for i in range(len(ray_list_of_k_sample_indices)):
+        ray_list_of_k_sample_indices[i] = np.array(ray_list_of_k_sample_indices[i])
+
+    ## load the ray_full_express if it's hdf5
+    if hdf5_file is not None:
+        h5f = h5py.File(hdf5_file, 'r')
+        ray_full_expression=h5f["infile"]
+
+    ##################
+    ## global vars
+    global_percent_express_dict = {index:None for index in indices}
+    global_non_zero_mean_dict = {index:None for index in indices}
+    ##################
+    ## group vars
+    group_percent_express_dict = {index:None for index in indices}
+    group_non_zero_mean_dict = {index:None for index in indices}
+    ##################
+    ## aov vars
+    aov_dict = {index:None for index in indices}
+    ##################
+
+    temp_array_start = min_index
+    temp_array_end = -99
+    subset_interval = 100
+    #for index in indices:
+    while temp_array_end < max_index:
+        #########################
+        ## get the interval
+        temp_array_end = min([temp_array_start+subset_interval,max_index])
+
+        ## subset the table 
+        in_mem_express = np.array(ray_full_expression[temp_array_start:temp_array_end,:])
+
+        #if temp_array_start % 200 == 0:
+        if True:
+            print("worker is:",(temp_array_start-min_index)/len(indices)*100," percent done")
+
+        #########################
+        ## global express
+        temp_binary_int = np.array(in_mem_express > 0,dtype=int)
+        #temp_binary_int = np.array(deepcopy(temp_binary),dtype=int)
+        #temp_num_express = np.sum(temp_binary_int)
+        temp_num_express = np.sum(temp_binary_int, axis = 1)
+        global_percent_express = temp_num_express/temp_binary_int.shape[1]
+        global_non_zero_mean = np.sum(in_mem_express,axis=1)/temp_num_express
+        #
+        idx_range = range(temp_array_start, temp_array_end)
+        for i in range(in_mem_express.shape[0]):
+            index=idx_range[i]
+            global_percent_express_dict[index] = global_percent_express[i]
+            global_non_zero_mean_dict[index] = global_non_zero_mean[i]
+
+        #########################
+        ## group vars
+        group_percent_express = np.zeros((in_mem_express.shape[0],len(ray_list_of_k_sample_indices)))
+        group_non_zero_mean = np.zeros((in_mem_express.shape[0],len(ray_list_of_k_sample_indices)))
+        for i in range(len(ray_list_of_k_sample_indices)):
+            ## subset
+            temp_binary_int_subset = temp_binary_int[:,ray_list_of_k_sample_indices[i]]
+            ##
+            temp_num_express_subset = np.sum(temp_binary_int_subset, axis=1)
+            #print(temp_num_express_subset)
+            #print(ray_list_of_k_sample_indices[i].shape[0])
+            #print(group_percent_express[:,i])
+            group_percent_express[:,i] = temp_num_express_subset/ray_list_of_k_sample_indices[i].shape[0]## it's a 1 dim array, so this is how many cells are in the group
+            group_non_zero_mean[:,i] = np.sum(in_mem_express[:,ray_list_of_k_sample_indices[i]])/temp_num_express_subset
+        
+        for i in range(in_mem_express.shape[0]):
+            index=idx_range[i]
+            group_percent_express_dict[index] = group_percent_express[i,:].tolist()
+            group_non_zero_mean_dict[index] = group_non_zero_mean[i,:].tolist()
+        
+        ##################
+        ## aov vars
+        for i in range(in_mem_express.shape[0]):
+            index=idx_range[i]
+            if np.sum(temp_binary_int[i,:]) == 0:
+                aov_dict[index] = [np.nan,np.nan]
+            list_of_group_values=[]
+            for group in range(0,len(ray_list_of_k_sample_indices)):
+                list_of_group_values.append(in_mem_express[i,ray_list_of_k_sample_indices[group]])
+                
+            return_val = list(aov(*list_of_group_values))
+            aov_dict[index] = return_val
+
+        #################
+        ## index incrementing 
+        temp_array_start=deepcopy(temp_array_end)
+
+    if hdf5_file is not None:
+        h5f.close()
+
+    return(global_percent_express_dict, 
+           global_non_zero_mean_dict,
+           group_percent_express_dict,
+           group_non_zero_mean_dict,
+           aov_dict)
+
+
+
 @ray.remote
 def ray_get_global_express_stats(indices, 
                                  ray_full_expression,
@@ -552,9 +664,7 @@ if not one_group:
     ratio_non_zero_mean_expression = np.zeros((full_expression.shape[0], len(list_of_k_sample_indices))) ## ratio of within cell type percent expression to global percent expression
     
     # #######################################################################################
-    # ################## OLD ##############################
-    # #####################################################
-    # ## first do the global expression calculations
+
     if args.processes == 1:
         def get_global_express_stats(index):
             global full_expression
@@ -617,17 +727,11 @@ if not one_group:
 
         print(k_group_percent_express)
         print(non_zero_mean_expression)
-        
-
-        ######################## /OLD #####################################
-
-    #################### /OLD ###########################
-    #################### NEW ############################
     else:
         if args.processes == None:
             args.processes = multiprocessing.cpu_count()
         threads = args.processes
-        indices_list = get_indices(threads, full_expression.shape[0])
+        indices_list = get_contiguous_indices(threads, full_expression.shape[0])
         
         ray.init()
 
@@ -645,78 +749,44 @@ if not one_group:
         r_jobs = []
         for t in range(threads):
             if not args.hdf5:
-                r_jobs.append(ray_get_global_express_stats.remote(indices_list[t], 
-                                                                  ray_full_expression))
+                r_jobs.append(ray_get_global_cell_type_aov_stats.remote(indices_list[t], 
+                                                                        ray_full_expression,
+                                                                        list_of_k_sample_indices))
             else:
-                r_jobs.append(ray_get_global_express_stats.remote(indices_list[t], 
-                                                                  None,
-                                                                  hdf5_file = hdf5_file_list[t]))
+                r_jobs.append(ray_get_global_cell_type_aov_stats.remote(indices_list[t], 
+                                                                        None,
+                                                                        list_of_k_sample_indices,
+                                                                        hdf5_file = hdf5_file_list[t]))
         temp_r_results = ray.get(r_jobs)
         ray.shutdown()
+        ########################################
     
         global_percent_express_dict_list = []
         global_non_zero_mean_dict_list = []
+        k_group_percent_express_dict_list = []
+        non_zero_mean_expression_dict_list = []
+        temp_aov_results = []
 
         for t in range(len(temp_r_results)):
+            # global_percent_express_dict, 
+            # global_non_zero_mean_dict,
+            # group_percent_express_dict,
+            # group_non_zero_mean_dict,
+            # aov_dict
             global_percent_express_dict_list.append(temp_r_results[t][0])
             global_non_zero_mean_dict_list.append(temp_r_results[t][1])
+            k_group_percent_express_dict_list.append(temp_r_results[t][2])
+            non_zero_mean_expression_dict_list.append(temp_r_results[t][3])
+            temp_aov_results.append(temp_r_results[t][4])
 
         global_percent_express = ray_dicts_to_array(global_percent_express_dict_list)
         global_non_zero_mean = ray_dicts_to_array(global_non_zero_mean_dict_list)
+        k_group_percent_express = ray_dicts_to_array(k_group_percent_express_dict_list)
+        non_zero_mean_expression = ray_dicts_to_array(non_zero_mean_expression_dict_list)
+        all_aov_results = ray_dicts_to_array(temp_aov_results).tolist()
 
         global_percent_express = global_percent_express[:,0]
         global_non_zero_mean = global_non_zero_mean[:,0]
-
-        print(global_percent_express)
-        print(global_non_zero_mean)
-        #################### /NEW ############################    
-        #######################################################################################
-
-        print("calculating percent expressed in cell types and mean non-zero expression")
-
-        
-        ######################## NEW #####################################
-        ray.init()
-        if not args.hdf5:
-            ray_full_expression = ray.put(full_expression)
-
-        ###########
-        # indices,
-        # ray_full_expression,
-        # ray_list_of_k_sample_indices,
-        # hdf5_file = None)
-
-        r_jobs = []
-
-        if args.processes == None:
-            args.processes = multiprocessing.cpu_count()
-        threads = args.processes
-        for t in range(threads):
-            if not args.hdf5:
-                r_jobs.append(ray_get_cell_type_express_stats.remote(indices_list[t], 
-                                                                     ray_full_expression,
-                                                                     list_of_k_sample_indices))
-            else:
-                r_jobs.append(ray_get_cell_type_express_stats.remote(indices_list[t], 
-                                                                     None,
-                                                                     list_of_k_sample_indices,
-                                                                     hdf5_file = hdf5_file_list[t]))
-        temp_r_results = ray.get(r_jobs)
-        ray.shutdown()
-        
-        k_group_percent_express_dict_list = []
-        non_zero_mean_expression_dict_list = []
-
-        for t in range(len(temp_r_results)):
-            k_group_percent_express_dict_list.append(temp_r_results[t][0])
-            non_zero_mean_expression_dict_list.append(temp_r_results[t][1])
-
-        k_group_percent_express = ray_dicts_to_array(k_group_percent_express_dict_list)
-        non_zero_mean_expression = ray_dicts_to_array(non_zero_mean_expression_dict_list)
-
-        print(k_group_percent_express)
-        print(non_zero_mean_expression)
-
         ######################## /NEW #####################################
 
     ## calculate ratios
@@ -813,8 +883,6 @@ if not one_group:
 ##################################################################################################################
 ##### do a one way anova between groups to test for variables that are significantly different between them ######
 
-from scipy.stats.mstats import f_oneway as aov
-from scipy.stats import ttest_ind as ttest
 
 def get_ttests(index, aov_p):
     ## for use in the context of a Friedman's Protected LSD post-hoc to significant anovas
@@ -847,13 +915,6 @@ def get_anova(index):
     return_val = list(aov(*list_of_group_values))
     return(return_val)
 
-# if not very_big_file:
-#     if len(list_of_k_sample_indices) > 1:
-#         enough_k_samples = True
-#     else:
-#         enough_k_samples = False
-# else:
-#     enough_k_samples = False
 
 all_sig_enriched_files=[]
 all_sig_enriched_gprof_files = []
@@ -877,35 +938,6 @@ if not one_group:
         print(all_aov_results[:5])
         ########## /OLD ######################
     else:
-        ######### NEW ##################
-        ray.init()
-        if not args.hdf5:
-            ray_full_expression = ray.put(full_expression)
-        ###########
-        # indices,
-        # ray_full_expression,
-        # ray_list_of_k_sample_indices,
-        # hdf5_file = None)
-
-        r_jobs = []
-
-        for t in range(threads):
-            if not args.hdf5:
-                r_jobs.append(ray_get_anova.remote(indices_list[t],
-                                                   ray_full_expression,
-                                                   list_of_k_sample_indices))
-            else:
-                r_jobs.append(ray_get_anova.remote(indices_list[t], 
-                                                   None,
-                                                   list_of_k_sample_indices,
-                                                   hdf5_file = hdf5_file_list[t]))
-
-
-        temp_aov_results = ray.get(r_jobs)
-        ray.shutdown()
-            
-        all_aov_results = ray_dicts_to_array(temp_aov_results).tolist()
-        
         ## clean up the copied files
         if args.hdf5:
             for file in hdf5_file_list:
